(function() {
    "use strict";

    angular.module("weChatApp")
        .controller("dashboardCtrl", ["$scope", "$timeout", "$q", "_currentUser", "_unreachableObjects", "DashboardHelper",
            "TranslationService", "SentMessageService", "TerritoryService", "ConsentService", "TsfService", "Utility",
            "ChartJsPlugins", "AppConstants", "Queries", DashboardCtrl])
    ;

    function DashboardCtrl($scope, $timeout, $q, _currentUser, _unreachableObjects, dashboardHelper,
                           translationService, sentMessageService, territoryService, consentService, tsfService, utility,
                           chartJsPlugins, appConstants, queries) {
        var dash = this;
        var OBJECT_LABELS = {};
        var FIELD_LABELS = {};
        var PICKLIST_LABELS = {};
        var user_timezone;
        var user_locale;

        dashboardHelper.excludeUnreachableFieldFromQueries(queries, _unreachableObjects);
        dashboardHelper.getLabels(_unreachableObjects).then(function(resp) {
            OBJECT_LABELS = resp.objectLabels;
            FIELD_LABELS = resp.fieldLabels;
            PICKLIST_LABELS = resp.picklistLabels;
            preStartSetup();
            start();
        });

        function preStartSetup() {
            user_timezone = _currentUser.TimeZoneSidKey;
            user_locale = _currentUser.LocaleSidKey;

            if(!moment.tz.zone(user_timezone)) {
                user_timezone = "GMT";
            }

            moment.locale("en");
            if(user_locale) {
                moment.locale(user_locale); // moment will ignore the locale here if it's not valid
            }

            dash.detailMode = false;
        }

        function start() {
            initGlobalDateFilter();

            if(dashboardHelper.okToEnableTile(appConstants.SENT_MESSAGE, _unreachableObjects)) {
                getSentMessages(getFilterDates());
            }
            if(dashboardHelper.okToEnableTile(appConstants.CONSENT, _unreachableObjects)) {
                getConsents();
            }
        }

        function initGlobalDateFilter() {
            dash.globalDateFilterOptions = [];
            dash.globalDateFilterOptions.push({
                label: translationService.getMessage.apply(null, ["LAST_DAYS", 30]),
                dayRange: -30
            });
            dash.globalDateFilterOptions.push({
                label: translationService.getMessage.apply(null, ["LAST_DAYS", 60]),
                dayRange: -60
            });
            dash.globalDateFilterOptions.push({
                label: translationService.getMessage.apply(null, ["LAST_DAYS", 90]),
                dayRange: -90
            });
            dash.selectedDateFilter = dash.globalDateFilterOptions[0];
        }

        function getFilterDates() {
            var backward = dash.selectedDateFilter.dayRange !== Math.abs(dash.selectedDateFilter.dayRange);
            var dayRange = Math.abs(dash.selectedDateFilter.dayRange);

            var filterEndDate;
            var filterStartDate;
            if(backward) {
                filterEndDate = moment(); // now
                filterStartDate = filterEndDate.clone().subtract(dayRange, "days");
            } else {
                filterStartDate = moment(); // now
                filterEndDate = filterStartDate.clone().add(dayRange, "days");
            }

            var startDateUTC = moment.utc(filterStartDate);
            var endDateUTC = moment.utc(filterEndDate);
            return {
                startDatetime: startDateUTC.toISOString(),
                endDatetime: endDateUTC.toISOString(),
                startDate: startDateUTC.format(appConstants.YEAR_MONTH_DAY_FORMAT),
                endDate: endDateUTC.format(appConstants.YEAR_MONTH_DAY_FORMAT),
                startDateMoment: startDateUTC,
                endDateMoment: endDateUTC
            };
        }

        function getSentMessages(filterDates) {
            var includeAccountName = true;
            var includeMultichannelContent = true;
            var includeContentType = true;
            if(!!_unreachableObjects.Sent_Message_vod__c) {
                includeAccountName = !~(_unreachableObjects.Sent_Message_vod__c.indexOf("Account"));
                includeMultichannelContent = !~(_unreachableObjects.Sent_Message_vod__c.indexOf("Multichannel_Content_vod__c"));
                includeContentType = !~(_unreachableObjects.Sent_Message_vod__c.indexOf("Content_Type_vod__c"));
            }

            sentMessageService.getSentMessageByUserId(_currentUser.Id, filterDates, includeAccountName,
                includeMultichannelContent, includeContentType).then(function(sentMessageResp) {
                console.info(sentMessageResp);

                var openedSMs = sentMessageResp.filter(function(sm) {
                    // for WM discrepancy
                    if(typeof sm.Opened_vod__c === "string") {
                        return !!parseInt(sm.Opened_vod__c);
                    }
                    return !!sm.Opened_vod__c;
                });

                dash.sentMessageResp = sentMessageResp;
                dash.messageOpenRate = 0;

                if(sentMessageResp.length > 0) {
                    dash.messageOpenRate = (100 * openedSMs.length / sentMessageResp.length).toFixed();
                }
                dash.sentMessageReady = true;
            });
        }

        function getConsents() {
            dash.hcpOptInPercentage = 0;
            dash.hcpSentMessageCoverPercentage = 0;

            // Get accounts under user territory
            territoryService.getAccountsUnderUserTerritory(_currentUser.Id).then(function(accountResp) {
                // Prepare a list of TSF external ids to use later
                // TSF external id is formatted as "AccountID__TerritoryName"
                var tsfExtIds = [];
                accountResp.forEach(function(acct) {
                    acct.territories.forEach(function(acctTerritory) {
                        tsfExtIds.push(acct.Id + "__" + acctTerritory.Name);
                    });
                });

                // Get target accounts from TSF table using TSF external ids
                var targetAccounts = [];
                var targetAccountTracker = {};
                tsfService.getTargetAccountByExternalIds(tsfExtIds).then(function(tsfResp) {
                    tsfResp.forEach(function(tsf) {
                        if(!targetAccountTracker[tsf.Account_vod__c]) {
                            targetAccounts.push(_.find(accountResp, function(acct) {
                                return acct.Id === tsf.Account_vod__c;
                            }));
                            targetAccountTracker[tsf.Account_vod__c] = true;
                        }
                    });
                    console.info(targetAccounts);

                    // Get WeChat Consent by targeted accounts
                    /** Include content type query if
                     *  1. has OLS on Content_Type_vod__c (i.e. _unreachableObjects.Content_Type_vod__c is undefined)
                     *  2. has FLS on Content_Type_vod__c.Name (i.e. _unreachableObjects.Content_Type_vod__c does not contain "Name")
                     */
                    var includeContentType = true;
                    if(!!_unreachableObjects.Content_Type_vod__c) {
                        includeContentType = _unreachableObjects.Content_Type_vod__c.length !== 0 && !~(_unreachableObjects.Content_Type_vod__c.indexOf("Name"));
                    }
                    consentService.getWeChatConsentByAccounts(targetAccounts, includeContentType).then(function(consentResp) {
                        console.info(consentResp);
                        dash.consentResp = consentResp;

                        /** To be considered as opt-in consent, the record need to
                         *  1. Opt_Type_vod__c = Opt_In_vod
                         *  2. Opt_Expiration_Date_vod__c >= TODAY OR EMPTY
                         */
                        var activeOptInConsent = consentResp.filter(function(c) {
                            return c.Opt_Type_vod__c === "Opt_In_vod" && (!c.Opt_Expiration_Date_vod__c ||
                                utility.parseStringToDateValue(c.Opt_Expiration_Date_vod__c) >= moment.utc().valueOf());
                        });
                        var targetAccountsWithActiveOptInConsent = _.uniqBy(activeOptInConsent, "Account_vod__c");

                        if(targetAccounts.length > 0) {
                            dash.hcpOptInPercentage = (100 * targetAccountsWithActiveOptInConsent.length / targetAccounts.length).toFixed();
                        }

                        $scope.$broadcast(appConstants.UPDATE_DOUGHNUT_CHART_EVENT, {
                            name: "OPT_IN_RATE",
                            data: dash.hcpOptInPercentage
                        });
                    });

                    // Get Sent Messages for targeted user
                    sentMessageService.getSentMessageByUserId(_currentUser.Id).then(function(allSentMessages) {
                        var targetAccountsWithSentMessage = _.uniqBy(allSentMessages, "Account_vod__c").filter(function(sm) {
                            // target accounts must match account from the right territory
                            return !!_.find(targetAccounts, function(account) {
                                return account.Id === sm.Account_vod__c;
                            });
                        });

                        if(targetAccounts.length > 0) {
                            dash.hcpSentMessageCoverPercentage = (100 * targetAccountsWithSentMessage.length / targetAccounts.length).toFixed();
                        }

                        $scope.$broadcast(appConstants.UPDATE_DOUGHNUT_CHART_EVENT, {
                            name: "SENT_MESSAGE_COVERAGE",
                            data: dash.hcpSentMessageCoverPercentage
                        });
                    });

                    dash.consentReady = true;
                });
            });
        }

        dash.globalFilterOnChange = function() {
            if(dashboardHelper.okToEnableTile(appConstants.SENT_MESSAGE, _unreachableObjects)) {
                getSentMessages(getFilterDates());
            }
        };

        dash.toggleDetailMode = function() {
            dash.detailMode = !dash.detailMode;
            if(!dash.detailMode) {
                dash.showSentMessageDetail = false;
                dash.showConsentDetail = false;
            }
        };

        dash.viewAllSentMessages = function() {
            var fieldLabels = FIELD_LABELS.Sent_Message_vod__c;
            dash.sentMessageTableHeaders = [{
                label: fieldLabels.Content_vod__c,
                name: "Content_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Account_vod__c,
                name: "Account_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Sent_Datetime_vod__c,
                name: "Sent_Datetime_vod__c",
                isDate: true,
                defaultSort: true
            }, {
                label: translationService.getMessage('OPENED'),
                name: "Status",
                isDate: false
            }, {
                label: fieldLabels.Last_Open_Datetime_vod__c,
                name: "Last_Open_Datetime_vod__c",
                isDate: true
            }, {
                label: fieldLabels.Total_Duration_vod__c,
                name: "Total_Duration_vod__c",
                isDate: false
            }, {
                label: translationService.getMessage('SERVICE_ACCOUNT'),
                name: "Service_Account_vod_del__c",
                isDate: false
            }];

            dash.sentMessageTableBody = [];
            dash.sentMessageInfoCard = [];
            dash.sentMessageResp.forEach(function(sm) {
                dash.sentMessageTableBody.push({
                    Content_vod__c: sm.contentName,
                    Account_vod__c: sm.accountName,
                    Sent_Datetime_vod__c: utility.parseStringToDateValue(sm.Sent_Datetime_vod__c, appConstants.YEAR_MONTH_DAY_FORMAT),
                    Status: sm.Last_Open_Datetime_vod__c ? translationService.getMessage("OPENED") : translationService.getMessage("UNOPENED"),
                    Last_Open_Datetime_vod__c: utility.parseStringToDateValue(sm.Last_Open_Datetime_vod__c, appConstants.YEAR_MONTH_DAY_FORMAT),
                    Total_Duration_vod__c: sm.Total_Duration_vod__c,
                    Service_Account_vod_del__c: sm.serviceAccountName
                });
                // fill up the info card for detail view (iPhone)
                dash.sentMessageInfoCard.push(_.assign({
                    cardHeader: OBJECT_LABELS.Sent_Message_vod__c.singular + " " + dash.sentMessageTableBody.length
                }, _.last(dash.sentMessageTableBody)));
            });

            // sort the info card in descending order by Sent Datetime
            dash.sentMessageInfoCard.sort(function(sm1, sm2) {
                if(sm1.Sent_Datetime_vod__c && sm2.Sent_Datetime_vod__c) {
                    return sm2.Sent_Datetime_vod__c - sm1.Sent_Datetime_vod__c;
                } else if (sm1.Sent_Datetime_vod__c) {
                    return -1;
                } else if (sm2.Sent_Datetime_vod__c) {
                    return 1;
                } else {
                    return 0;
                }
            });

            dash.sentMessageObjectLabel = OBJECT_LABELS.Sent_Message_vod__c.plural;
            dash.showSentMessageDetail = true;
            dash.toggleDetailMode();
        };

        dash.viewAllConsents = function() {
            var fieldLabels = FIELD_LABELS.Multichannel_Consent_vod__c;
            dash.consentTableHeaders = [{
                label: fieldLabels.Account_vod__c,
                name: "Account_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Opt_Type_vod__c,
                name: "Opt_Type_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Capture_Datetime_vod__c,
                name: "Capture_Datetime_vod__c",
                isDate: true,
                defaultSort: true
            }, {
                label: translationService.getMessage('SERVICE_ACCOUNT'),
                name: "ServiceAccount",
                isDate: false
            }, {
                label: fieldLabels.Opt_Expiration_Date_vod__c,
                name: "Opt_Expiration_Date_vod__c",
                isDate: true
            }];
            dash.consentTableBody = [];
            dash.consentInfoCard = [];
            dash.consentResp.forEach(function(c) {
                dash.consentTableBody.push({
                    Account_vod__c: c.accountName,
                    Opt_Type_vod__c: PICKLIST_LABELS.Multichannel_Consent_vod__c.Opt_Type_vod__c[c.Opt_Type_vod__c],
                    Capture_Datetime_vod__c: utility.parseStringToDateValue(c.Capture_Datetime_vod__c, appConstants.FULL_DATE_TIME_FORMAT),
                    ServiceAccount: c.contentName,
                    Opt_Expiration_Date_vod__c: utility.parseStringToDateValue(c.Opt_Expiration_Date_vod__c, appConstants.YEAR_MONTH_DAY_FORMAT)
                });
            });

            // we only contain the 50 latest consent by capture datetime for the detail view
            dash.consentTableBody.sort(function(c1, c2) {
                if(c1.Capture_Datetime_vod__c && c2.Capture_Datetime_vod__c) {
                    return c2.Capture_Datetime_vod__c - c1.Capture_Datetime_vod__c;
                } else if (c1.Capture_Datetime_vod__c) {
                    return -1;
                } else if (c2.Capture_Datetime_vod__c) {
                    return 1;
                } else {
                    return 0;
                }
            });
            dash.consentTableBody.splice(50);

            // fill up the info card for detail view (iPhone)
            dash.consentTableBody.forEach(function(c, i) {
                dash.consentInfoCard.push(_.assign({
                    cardHeader: OBJECT_LABELS.Multichannel_Consent_vod__c.singular + " " + (++i)
                }, c));
            });

            dash.consentObjectLabel = OBJECT_LABELS.Multichannel_Consent_vod__c.plural;
            dash.showConsentDetail = true;
            dash.toggleDetailMode();
        };

        // register ChartJS custom plug-ins
        Chart.pluginService.register(chartJsPlugins.centerLabel);
    }
}());